@@ -1,13 +1,14 @@
 from typing import Any, Dict, List
 import requests
 from requests import HTTPError, RequestException
+from requests.adapters import HTTPAdapter
+from urllib3.util.retry import Retry
 import logging
 import os
 import shutil
 from pathlib import Path
 import psycopg2
 from psycopg2.extras import execute_values
-import sys
 from redmail import EmailSender
 
 try:
@@ -21,12 +22,14 @@ proxies = {
 }
 
 
-def general_request(method: str, url: str, **kwargs) -> str:
+def general_request(method: str, url: str, retries=3, **kwargs) -> str:
     """Esegue una richiesta custom ad un url, utilizzando il metodo request
+    Se ci sono errori nell'API, vengono fatti dei retry (default 3)
 
     Args:
         method (str): metodo (POST, GET, etc...)
         url (str): url da interrogare
+        retries (int): default a 3, numero di retry da fare
 
     Raises:
         ConnectionError: Errore di rete
@@ -37,14 +40,29 @@ def general_request(method: str, url: str, **kwargs) -> str:
     Returns:
         str: contenuto della risposta
     """
+    retries = retries
+    session = requests.Session()
+    retry = Retry(
+        total=retries,
+        read=retries,
+        connect=retries,
+        backoff_factor=0.3,
+        method_whitelist=False,
+        raise_on_status=True,
+    )
+    adapter = HTTPAdapter(max_retries=retry)
+    session.mount("http://", adapter)
+    session.mount("https://", adapter)
     try:
-        r = requests.request(method, url, proxies=proxies, **kwargs)
+        r = session.request(method, url, proxies=proxies, **kwargs)
         r.raise_for_status()
         logging.info("API called")
         return r.text
     except ConnectionError as exc:
         error_type = str(exc)
-        logging.error(f'Netowork error (DNS, refused connection, etc) - {method} {url} {kwargs}')
+        logging.error(
+            f"Netowork error (DNS, refused connection, etc) - {method} {url} {kwargs}"
+        )
         raise Exception("Netowork error (DNS, refused connection, etc)")
     except HTTPError as exc:
         error_type = str(exc)
@@ -56,7 +74,9 @@ def general_request(method: str, url: str, **kwargs) -> str:
         raise Exception("Timeout error from API")
     except RequestException as exc:
         error_type = str(exc)
-        logging.error(f"General error in getting data from API - {method} {url} {kwargs}")
+        logging.error(
+            f"General error in getting data from API - {method} {url} {kwargs}"
+        )
         raise Exception("General error in getting data from API")
 
 
@@ -107,7 +127,9 @@ def __create_sql_query(table_name, data: List[Dict[str, Any]]) -> str:
         str: query
     """
     if not isinstance(table_name, str):
-        logging.error(f"The name of the table must be a string, and {table_name} is a {type(table_name)}")
+        logging.error(
+            f"The name of the table must be a string, and {table_name} is a {type(table_name)}"
+        )
         raise ValueError(
             f"The name of the table must be a string, and {table_name} is a {type(table_name)}"
         )
@@ -123,7 +145,7 @@ def insert_on_db(
     database: str,
     table_name: str,
     data: List[Dict[str, Any]],
-    truncate = False
+    truncate=False,
 ) -> None:
     """Genera la connessione al db PGSQL specificato ed esegue la query.
     Lancia un'eccezione in caso di errori lato db.
@@ -149,7 +171,7 @@ def insert_on_db(
         logging.info(f"Connected to {database} on {host}")
         cursor = connection.cursor()
         if truncate:
-            cursor.execute(f'TRUNCATE {table_name}')
+            cursor.execute(f"TRUNCATE {table_name}")
         values = [[value for value in row.values()] for row in data]
         execute_values(cursor, __create_sql_query(table_name, data), values)
         connection.commit()
